pub struct GasCharger { /* private fields */ }Expand description
Encapsulates the gas metering state (SuiGasStatus) and the payment source metadata,
whether it is from a smashed list (coin objects or address-balance withdrawals) or
un-metered. In other words, this serves the point of interaction between the on-chain data
(coins and address balances) and the gas meter.
Implementations
Sourceimpl GasCharger
impl GasCharger
pub fn new( tx_digest: TransactionDigest, payment_kind: PaymentKind, gas_status: SuiGasStatus, temporary_store: &mut TemporaryStore<'_>, protocol_config: &ProtocolConfig, ) -> Self
pub fn new_unmetered(tx_digest: TransactionDigest) -> Self
pub fn override_gas_charge_location( &mut self, location: PaymentLocation, ) -> Result<(), ExecutionError>
Sourcepub fn gas_payment_amount(&self) -> Option<GasPayment>
pub fn gas_payment_amount(&self) -> Option<GasPayment>
Return the amount available at the given input payment location. For unmetered, this is None. For smashed gas payments, this is the payment location and the total amount smashed. This information feels a bit brittle but should be used only by PTB execution. This might also differ from the final charge location, if override_gas_charge_location is used.
pub fn gas_budget(&self) -> u64
pub fn unmetered_storage_rebate(&self) -> u64
pub fn no_charges(&self) -> bool
pub fn is_unmetered(&self) -> bool
pub fn move_gas_status(&self) -> &GasStatus
pub fn move_gas_status_mut(&mut self) -> &mut GasStatus
pub fn into_gas_status(self) -> SuiGasStatus
pub fn summary(&self) -> GasCostSummary
pub fn track_storage_mutation( &mut self, object_id: ObjectID, new_size: usize, storage_rebate: u64, ) -> u64
pub fn reset_storage_cost_and_rebate(&mut self)
pub fn charge_publish_package( &mut self, size: usize, ) -> Result<(), ExecutionError>
pub fn charge_upgrade_package( &mut self, size: usize, ) -> Result<(), ExecutionError>
pub fn charge_input_objects( &mut self, temporary_store: &TemporaryStore<'_>, ) -> Result<(), ExecutionError>
pub fn charge_coin_transfers( &mut self, protocol_config: &ProtocolConfig, num_non_gas_coin_owners: u64, ) -> Result<(), ExecutionError>
Sourcepub fn reset(&mut self, temporary_store: &mut TemporaryStore<'_>)
pub fn reset(&mut self, temporary_store: &mut TemporaryStore<'_>)
Resets any mutations, deletions, and events recorded in the store, as well as any storage costs and rebates, then Re-runs gas smashing. Effects on store are now as if we were about to begin execution
Sourcepub fn charge_gas<T, E: ExecutionErrorTrait>(
&mut self,
temporary_store: &mut TemporaryStore<'_>,
execution_result: &mut Result<T, E>,
) -> GasCostSummary
pub fn charge_gas<T, E: ExecutionErrorTrait>( &mut self, temporary_store: &mut TemporaryStore<'_>, execution_result: &mut Result<T, E>, ) -> GasCostSummary
Entry point for gas charging.
- Compute tx storage gas costs and tx storage rebates, update storage_rebate field of mutated objects
- Deduct computation gas costs and storage costs, credit storage rebates. The happy path of this function follows (1) + (2) and is fairly simple. Most of the complexity is in the unhappy paths:
- if execution aborted before calling this function, we have to dump all writes + re-smash gas, then charge for storage
- if we run out of gas while charging for storage, we have to dump all writes + re-smash gas, then charge for storage again